@@ -30,18 +30,18 @@
 # include	<X11/Xos.h>
 # include	<X11/Xaw/Command.h>
 # include	<X11/Xaw/Box.h>
-# include	<X11/Xaw/Dialog.h>
-# include	<X11/Xaw/Label.h>
-# include	<X11/Xaw/MenuButton.h>
-# include	<X11/Xaw/SimpleMenu.h>
-# include	<X11/Xaw/SmeBSB.h>
-# include	<X11/Xaw/AsciiText.h>
 # include	<X11/Xaw/Cardinals.h>
+# include	<Xkw/KLabel.h>
+# include	<Xkw/KCommand.h>
+# include	<Xkw/KMenuButton.h>
+# include	<Xkw/KSimpleMenu.h>
+# include	<Xkw/KSmeBSB.h>
 # include	<Xkw/Cards.h>
 # include	<Xkw/Layout.h>
 # include	<X11/Xutil.h>
 # include	<Xkw/CardsUtil.h>
 # include	<Xkw/Message.h>
+# include	"KAces-res.h"
 
 Widget	    toplevel;
 Widget	    frame;
@@ -62,6 +62,7 @@ Widget	    pileAll;
 #define NUM_STACKS  7
 #define NUM_PILES   4
 #define NUM_CARDS   52
+#define MAX_SCORE   340
 
 CardStackRec	deckStack;
 CardStackRec	stackStacks[NUM_STACKS];
@@ -69,8 +70,6 @@ CardStackRec	pileStacks[NUM_PILES];
 
 CardRec		rawcards[NUM_CARDS];
 
-CardStackPtr	fromStack;
-CardPtr		fromCard;
 int		dealNumber;
 
 typedef struct _acesResources {
@@ -188,6 +187,8 @@ Deal (void)
 	CardMove (&deckStack, deckStack.last, &stackStacks[col], True);
 	CardTurn (stackStacks[col].last, CardFaceUp, True);
     }
+    CardNextHistory ();
+    DisplayStacks ();
 }
 
 static void
@@ -196,8 +197,6 @@ NewGame (void)
     CardsRemoveAllCards (deck);
     CardsRemoveAllCards (piles);
     CardsRemoveAllCards (stacks);
-    fromStack = 0;
-    fromCard = 0;
     InitStacks ();
     GenerateCards ();
     CardShuffle (&deckStack, False);
@@ -217,8 +216,8 @@ Undo (void)
 static void
 Score (void)
 {
-    Message (message, "Current position scores %d out of 340.",
-	     ComputeScore ());
+    Message (message, "Current position scores %d out of %d.",
+	     ComputeScore (), MAX_SCORE);
 }
 
 static void
@@ -416,6 +415,13 @@ Play (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 	}
     }
     CardMove (from_stack, from_card, to_stack, True);
+    CheckStackTop (from_stack);
+    CardNextHistory ();
+    DisplayStacks ();
+    if (ComputeScore() == MAX_SCORE)
+        Message(message, "We have a winner!");
+    else
+        Message(message, "");
 }
 
 static Boolean
@@ -517,9 +523,6 @@ PileAll (void)
 	{
 	    Play (from_stack, from_card, to_stack);
 	    done = True;
-	    CheckStackTop (from_stack);
-	    CardNextHistory ();
-	    DisplayStacks ();
 	}
     } while (to_stack);
     if (!done)
@@ -556,80 +559,62 @@ Expand (CardStackPtr stack)
 
 /* Callbacks to user interface functions */
 
-static void
-StackCallback (Widget w, XtPointer closure, XtPointer data)
+static CardStackPtr
+WidgetToStack(Widget w, int col)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
-    CardPtr	    card;
-    String	    type;
-
-    (void) closure;
-    Message (message, "");
     if (w == stacks)
-	stack = &stackStacks[input->col];
-    else if (w == piles)
-	stack = &pileStacks[input->col];
-    else
-	return;
-    for (card = stack->last; card; card = card->prev)
-	if (card->isUp && card->row == input->row)
-	    break;
-    if (*input->num_params) {
-	type = *input->params;
-	if (!strcmp (type, "deck_source"))
-	{
-	    fromStack = stack;
-	    fromCard = stack->last;
-	}
-	else if (!strcmp (type, "stack_source"))
-	{
-	    fromStack = stack;
-	    fromCard = 0;
-	    if (!fromStack->last)
-		Message (message, "Selected stack is empty.");
-	}
-	else if (!strcmp (type, "card_source"))
-	{
-	    if (!card)
-		Message (message, "No card selected.");
-	    else
-	    {
-		fromStack = stack;
-		fromCard = card;
-	    }
-	}
-	else if (!strcmp (type, "dest"))
-	{
-	    if (fromStack)
-	    {
-		Play (fromStack, fromCard, stack);
-		CheckStackTop (fromStack);
-		fromStack = NULL;
-		CardNextHistory ();
-		DisplayStacks ();
-	    }
-	}
-	else if (!strcmp (type, "expand"))
-	{
-	    Expand (stack);
-	}
-    }
+	return &stackStacks[col];
+    if (w == piles)
+	return &pileStacks[col];
+    if (w == deck)
+	return &deckStack;
+    return NULL;
 }
 
 static void
-DeckCallback (Widget w, XtPointer closure, XtPointer data)
+InputCallback (Widget w, XtPointer closure, XtPointer data)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
+    HandInputPtr    input = (HandInputPtr) data;
+    CardStackPtr    stack = NULL;
+    CardStackPtr    startStack = NULL;
+    CardPtr	    card = NULL;
 
-    (void) w;
     (void) closure;
-    Message (message, "");
-    if (*input->num_params && !strcmp (*input->params, "deck_source"))
-    {
-	Deal ();
-	CardNextHistory ();
-	DisplayStacks ();
+    stack = WidgetToStack(w, input->current.col);
+    startStack = WidgetToStack(input->start.w, input->start.col);
+
+    if (!startStack || !stack)
+	return;
+
+    switch (input->action) {
+    case HandActionStart:
+	break;
+    case HandActionClick:
+        CardSetAnimate(True);
+        if (stack == &deckStack) {
+            Deal ();
+        } else if (stack->last) {
+            Play(stack, card, stack);
+        }
+        break;
+    case HandActionDrag:
+        CardSetAnimate(False);
+        if (stack == startStack)
+            break;
+        if (stack == &deckStack)
+            break;
+	if (startStack->last)
+	{
+            CardPtr     card = CardFromHandCard(input->start.private);
+	    Play (startStack, card, stack);
+	}
+	break;
+    case HandActionExpand:
+        Expand(stack);
+        break;
+    case HandActionUnexpand:
+        Message(message, "");
+        break;
     }
 }
 
@@ -808,11 +793,11 @@ CreateMenu (Widget parent, char *name, struct menuEntry *entries, int count)
     Widget  entry;
     int	    i;
 
-    menu = XtCreatePopupShell (name, simpleMenuWidgetClass,
+    menu = XtCreatePopupShell (name, ksimpleMenuWidgetClass,
 			       parent, NULL, ZERO);
     for (i = 0; i < count; i++) {
 	entry = XtCreateManagedWidget (entries[i].name,
-				       smeBSBObjectClass, menu, NULL, ZERO);
+				       ksmeBSBObjectClass, menu, NULL, ZERO);
 	XtAddCallback (entry, XtNcallback, entries[i].function, NULL);
     }
     return menu;
@@ -837,12 +822,9 @@ main (int argc, char **argv)
 {
     Atom wm_delete_window;
 
-#ifdef APPDEFAULTS
-    setenv("XAPPLRESDIR", APPDEFAULTS, 1);
-#endif
-
-    toplevel = XtInitialize (argv[0], "KAces", options, XtNumber(options),
-			     &argc, argv);
+    toplevel = XkwInitialize ("KAces", options, XtNumber(options),
+			      &argc, argv, True,
+			      defaultResources);
 
     XtGetApplicationResources (toplevel, (XtPointer)&acesResources, resources,
 			       XtNumber (resources), NULL, 0);
@@ -857,32 +839,32 @@ main (int argc, char **argv)
     frame = XtCreateManagedWidget ("frame", layoutWidgetClass, toplevel, NULL, 0);
     menuBar = XtCreateManagedWidget ("menuBar", layoutWidgetClass, frame, NULL, 0);
     fileMenuButton = XtCreateManagedWidget ("fileMenuButton",
-					    menuButtonWidgetClass,
+					    kmenuButtonWidgetClass,
 					    menuBar, NULL, ZERO);
     fileMenu = CreateMenu (fileMenuButton, "fileMenu",
 			   fileMenuEntries, XtNumber (fileMenuEntries));
-    newGame = XtCreateManagedWidget ("newGame", commandWidgetClass,
+    newGame = XtCreateManagedWidget ("newGame", kcommandWidgetClass,
 				     menuBar, NULL, ZERO);
     XtAddCallback(newGame, XtNcallback, NewGameCallback, NULL);
-    undo = XtCreateManagedWidget ("undo", commandWidgetClass,
+    undo = XtCreateManagedWidget ("undo", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(undo, XtNcallback, UndoCallback, NULL);
-    hint = XtCreateManagedWidget ("hint", commandWidgetClass,
+    hint = XtCreateManagedWidget ("hint", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(hint, XtNcallback, FindAMoveCallback, NULL);
-    score = XtCreateManagedWidget ("score", commandWidgetClass,
+    score = XtCreateManagedWidget ("score", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(score, XtNcallback, ScoreCallback, NULL);
-    pileAll = XtCreateManagedWidget ("pileAll", commandWidgetClass,
+    pileAll = XtCreateManagedWidget ("pileAll", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(pileAll, XtNcallback, PileAllCallback, NULL);
     deck = XtCreateManagedWidget ("deck", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (deck, XtNinputCallback, DeckCallback, NULL);
+    XtAddCallback (deck, XtNinputCallback, InputCallback, &deckStack);
     piles = XtCreateManagedWidget ("piles", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (piles, XtNinputCallback, StackCallback, NULL);
+    XtAddCallback (piles, XtNinputCallback, InputCallback, &pileStacks[0]);
     stacks = XtCreateManagedWidget ("stacks", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (stacks, XtNinputCallback, StackCallback, NULL);
-    message = XtCreateManagedWidget ("message", labelWidgetClass, frame, NULL, 0);
+    XtAddCallback (stacks, XtNinputCallback, InputCallback, &stackStacks[0]);
+    message = XtCreateManagedWidget ("message", klabelWidgetClass, frame, NULL, 0);
     srandom (getpid () ^ time ((long *) 0));
     NewGame ();
     XtRealizeWidget (toplevel);
@@ -890,6 +872,7 @@ main (int argc, char **argv)
 				   False);
     (void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
                             &wm_delete_window, 1);
+    XkwSetCardIcon(toplevel);
 
     XtMainLoop ();
 }
