@@ -17,34 +17,34 @@
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NCD.
  * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
+ * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  * Author:  Keith Packard, Network Computing Devices
  */
 
+# include	<stdlib.h>
+# include	<stdio.h>
 # include	<X11/Intrinsic.h>
 # include	<X11/StringDefs.h>
 # include	<X11/Shell.h>
 # include	<X11/Xos.h>
-# include	<X11/Xaw/Command.h>
+# include	<Xkw/KCommand.h>
 # include	<X11/Xaw/Box.h>
 # include	<X11/Xaw/Dialog.h>
-# include	<X11/Xaw/Label.h>
-# include	<X11/Xaw/MenuButton.h>
-# include	<X11/Xaw/SimpleMenu.h>
-# include	<X11/Xaw/SmeBSB.h>
+# include	<Xkw/KLabel.h>
+# include	<Xkw/KMenuButton.h>
+# include	<Xkw/KSimpleMenu.h>
+# include	<Xkw/KSmeBSB.h>
 # include	<X11/Xaw/AsciiText.h>
 # include	<X11/Xaw/Cardinals.h>
 # include	<Xkw/Cards.h>
 # include	<Xkw/Layout.h>
 # include	<X11/Xutil.h>
 # include	<Xkw/CardsUtil.h>
-
-#if defined(hpux) || defined(__hpux)
-#define srandom srand48
-#define random lrand48
-#endif
+# include	<Xkw/Message.h>
+# include	"KMontana-res.h"
+# include       "kgames.h"
 
 Widget	    toplevel;
 Widget	    frame;
@@ -66,6 +66,8 @@ Widget	    score;
 #define NUM_COLS    13
 #define NUM_CARDS   52
 
+#define MAX_SCORE 336
+
 CardStackRec	cardStacks[NUM_CARDS];
 CardStackRec	deckStack;
 CardRec		rawcards[NUM_CARDS];
@@ -92,11 +94,11 @@ MontanaResources montanaResources;
 #define ForAllCards	for (row = 0, stack = cardStacks; row < NUM_ROWS; row++) \
 			    for (col = 0; col < NUM_COLS; col++, stack++)
 
-InitStacks ()
+static void
+InitStacks (void)
 {
     ForAllCardVars;
-    CardDisplay	    display;
-    
+
     ForAllCards
     {
 	CardInitStack (stack, cards, CardsNone, True, row,
@@ -110,21 +112,20 @@ InitStacks ()
     currentScore = 0;
 }
 
-GenerateCards ()
+static void
+GenerateCards (void)
 {
-    int	    i;
-    
     CardGenerateStandardDeck (rawcards);
     deckStack.first = &rawcards[0];
     deckStack.last = &rawcards[NUM_CARDS-1];
 }
 
-RemoveAces (remember)
-    Boolean remember;
+static void
+RemoveAces (Boolean remember)
 {
     ForAllCardVars;
     int	    i;
-    
+
     i = 0;
     ForAllCards
     {
@@ -136,8 +137,8 @@ RemoveAces (remember)
     }
 }
 
-SetPosition (stack)
-    CardStackPtr    stack;
+static void
+SetPosition (CardStackPtr stack)
 {
     int	    i;
 
@@ -145,11 +146,18 @@ SetPosition (stack)
     position[i] = stack;
 }
 
-FirstDeal ()
+static int
+ComputeScore (void);
+
+static void
+UnHint (void);
+
+static void
+FirstDeal (void)
 {
     ForAllCardVars;
-    
-    ForAllCards 
+
+    ForAllCards
     {
 	CardMove (&deckStack, deckStack.last, stack, False);
 	SetPosition (stack);
@@ -159,9 +167,8 @@ FirstDeal ()
     (void) ComputeScore ();
 }
 
-CardStackPtr
-NotInOrder (row)
-    int	row;
+static CardStackPtr
+NotInOrder (int row)
 {
     CardStackPtr    stack;
     CardPtr	    card;
@@ -186,8 +193,8 @@ NotInOrder (row)
     return stack;
 }
 
-int
-ComputeScore ()
+static int
+ComputeScore (void)
 {
     int		    row;
     CardStackPtr    stack, notInOrder;
@@ -215,8 +222,9 @@ ComputeScore ()
 	score -= 100 * (dealNumber - NUM_DEALS);
     return score;
 }
-    
-DisplayStacks ()
+
+static void
+DisplayStacks (void)
 {
     ForAllCardVars;
 
@@ -236,15 +244,17 @@ EmptySavePtr	emptySave;
 
 #define RankScore(r)	((r) > Cards10 ? 10 : (int) (r))
 
-Boolean
-MontanaUndo ()
+static Boolean
+MontanaUndo (void)
 {
     EmptySavePtr    e;
     CardStackPtr    stack;
     CardsRank	    rank;
     CardsSuit	    suit;
-    
+
     e = emptySave;
+    if (!e)
+	return False;
     stack = e->empty;
     rank = stack->basePosition + 2;
     suit = stack->last->card.suit;
@@ -265,10 +275,11 @@ MontanaUndo ()
     emptyStacks[e->i] = e->empty;
     emptySave = e->next;
     Dispose (e);
+    return True;
 }
 
-MontanaDo (from_stack, to_stack)
-    CardStackPtr    from_stack, to_stack;
+static void
+MontanaDo (CardStackPtr from_stack, CardStackPtr to_stack)
 {
     EmptySavePtr    e;
     int		    i;
@@ -276,7 +287,7 @@ MontanaDo (from_stack, to_stack)
     CardsRank	    rank;
     CardsSuit	    suit;
     CardStackPtr    stack;
-    
+
     card = from_stack->last;
     CardMove (from_stack, card, to_stack, True);
     rank = to_stack->basePosition + 2;
@@ -315,16 +326,27 @@ MontanaDo (from_stack, to_stack)
 
 /* User interface functions */
 
-void
-ResetDealNumber (closure)
-    char    *closure;
+static void
+ResetDealNumber (void *closure)
 {
-    dealNumber = (int) closure;
+    dealNumber = (int) (intptr_t) closure;
     (void) ComputeScore ();
 }
 
-void
-Deal ()
+static void
+CheckWin(void)
+{
+    int score = ComputeScore();
+    if (dealNumber > NUM_DEALS)
+        score += 100 * (dealNumber - NUM_DEALS);
+    if (score == MAX_SCORE)
+        Message(message, "We have a winner!");
+    else
+        Message(message, "");
+}
+
+static void
+Deal (void)
 {
     ForAllCardVars;
     CardPtr	    card;
@@ -332,10 +354,7 @@ Deal ()
 
     UnHint ();
     if (dealNumber >= NUM_DEALS)
-    {
 	Message (message, "No more than %d redeals allowed.", NUM_DEALS);
-	return;
-    }
     stack = cardStacks;
     for (row = 0; row < NUM_ROWS; row++)
     {
@@ -360,20 +379,21 @@ Deal ()
     }
     RemoveAces (True);
     (void) ComputeScore ();
-    CardRecordHistoryCallback (ResetDealNumber, (char *) dealNumber);
+    CardRecordHistoryCallback (ResetDealNumber, (char *) (intptr_t) dealNumber);
 
     ++dealNumber;
     Message (message, "Dealt %d of %d.", dealNumber, NUM_DEALS);
     Message (dealDisplay, "Deal: %d", dealNumber);
-    
+
     CardSetAnimate (False);
     DisplayStacks ();
     CardSetAnimate (True);
     CardNextHistory ();
+    CheckWin();
 }
 
-void
-NewGame ()
+static void
+NewGame (void)
 {
     CardsRemoveAllCards (cards);
     fromStack = 0;
@@ -386,13 +406,12 @@ NewGame ()
     FirstDeal ();
     CardInitHistory ();
     DisplayStacks ();
-    Message (message, "Keith's Montana version 1.0");
+    Message (message, "Keith's Montana, Version " KGAMES_VERSION_STRING);
     Message (dealDisplay, "No deals.");
 }
 
-
-void
-Undo ()
+static void
+Undo (void)
 {
     if (!MontanaUndo ())
     {
@@ -405,22 +424,21 @@ Undo ()
     CardSetAnimate (True);
 }
 
-void
-Score ()
+static void
+Score (void)
 {
-    Message (message, "Current position scores %d out of 336.", 
-	     ComputeScore ());
+    Message (message, "Current position scores %d out of %d.",
+	     ComputeScore (), MAX_SCORE);
 }
 
-void
-Quit ()
+static void
+Quit (void)
 {
     exit (0);
 }
 
-CardStackPtr
-FindPlay (from_stack)
-    CardStackPtr    from_stack;
+static CardStackPtr
+FindPlay (CardStackPtr from_stack)
 {
     ForAllCardVars;
     CardPtr card = from_stack->last;
@@ -429,8 +447,8 @@ FindPlay (from_stack)
 	return NULL;
     if (card->card.rank == Cards2)
     {
-	for (row = 0, stack = cardStacks; 
-	     row < NUM_ROWS; 
+	for (row = 0, stack = cardStacks;
+	     row < NUM_ROWS;
 	     row++, stack += NUM_COLS)
 	{
 	    if (!stack->last)
@@ -441,7 +459,7 @@ FindPlay (from_stack)
     {
 	ForAllCards
 	{
-	    if (col < NUM_COLS - 1 && 
+	    if (col < NUM_COLS - 1 &&
 		!stack[1].last &&
 		stack->last && CardIsInSuitOrder (stack->last, card))
 	    {
@@ -452,9 +470,8 @@ FindPlay (from_stack)
     return NULL;
 }
 
-CardStackPtr
-FindReverse (to_stack)
-    CardStackPtr    to_stack;
+static CardStackPtr
+FindReverse (CardStackPtr to_stack)
 {
     ForAllCardVars;
     CardPtr card;
@@ -468,9 +485,8 @@ FindReverse (to_stack)
     return NULL;
 }
 
-CardStackPtr
-FindForward (from_stack)
-    CardStackPtr    from_stack;
+static CardStackPtr
+FindForward (CardStackPtr from_stack)
 {
     ForAllCardVars;
     CardPtr card;
@@ -484,12 +500,11 @@ FindForward (from_stack)
     return NULL;
 }
 
-Play (from_stack, to_stack)
-    CardStackPtr    from_stack, to_stack;
+static void
+Play (CardStackPtr from_stack, CardStackPtr to_stack)
 {
     int		    col;
     CardPtr	    card;
-    int		    i;
 
     UnHint ();
     card = from_stack->last;
@@ -527,38 +542,24 @@ Play (from_stack, to_stack)
 	}
 	if (col > 0 && !CardIsInSuitOrder (to_stack[-1].last, card))
 	{
-	    Message (message, "Can't move the %P next to the %P.", 
+	    Message (message, "Can't move the %P next to the %P.",
 		     &card->card, &to_stack[-1].last->card);
 	    return;
 	}
     }
     MontanaDo (from_stack, to_stack);
+    CheckWin();
 }
 
 #define MAX_MOVES   16
 
-PrintMove (m)
-    CardStackPtr    m[2];
-{
-    int	row = m[1]->position;
-    int col = m[1]->basePosition;
-    CardPtr card = m[0]->last;
-    
-    printf ("Move the %s of %s to %d,%d\n", 
-	    CardsRankName(card->card.rank), CardsSuitName(card->card.suit),
-	    row, col);
-}
-
-int
-FindAllMoves (moves)
-    CardStackPtr moves[MAX_MOVES][2];
+static int
+FindAllMoves (CardStackPtr moves[MAX_MOVES][2])
 {
     int			i, j, k;
     CardStackPtr	to_stack;
-    CardStackPtr	from_stack;
     CardPtr		card;
     CardsSuit		suit;
-    CardsRank		rank;
 
     j = 0;
     for (i = 0; i < 4; i++)
@@ -596,10 +597,8 @@ typedef struct _Solve {
     CardStackPtr    moves[MAX_MOVES][2];
 } SolveRec, *SolvePtr;
 
-Seek (s, bestp, levels)
-    SolvePtr	s;
-    int		*bestp;
-    int		levels;
+static int
+Seek (SolvePtr s, int *bestp, int levels)
 {
     SolveRec	ns;
     int		i, best;
@@ -633,8 +632,8 @@ Seek (s, bestp, levels)
 
 #define MAX_SEARCH   1000
 
-void
-BestMove ()
+static void
+BestMove (void)
 {
     SolveRec	s;
     int		score;
@@ -652,30 +651,30 @@ BestMove ()
     card = s.moves[best][0]->last;
     if (card->card.rank == Cards2)
 	Message (message, "Play the %P in column %d (%d)",
-		 card->card, s.moves[best][1]->position, score);
+		 &card->card, s.moves[best][1]->position, score);
     else
-	Message (message, "Play the %P (%d)", card->card, score);
+	Message (message, "Play the %P (%d)", &card->card, score);
 }
 
-UnHint ()
+static void
+UnHint (void)
 {
-    if (hintStack) 
+    if (hintStack)
     {
 	CardTurn (hintStack->last, CardFaceUp, True);
 	hintStack = 0;
     }
 }
 
-Hint (stack)
-    CardStackPtr    stack;
+static void
+Hint (CardStackPtr stack)
 {
     hintStack = stack;
     CardTurn (stack->last, CardFaceDown, True);
 }
 
-FindAMove (stack, forward)
-    CardStackPtr    stack;
-    Boolean	    forward;
+static void
+FindAMove (CardStackPtr stack, Boolean forward)
 {
     CardStackPtr    hint_stack;
 
@@ -688,12 +687,14 @@ FindAMove (stack, forward)
 	Hint (hint_stack);
 }
 
-Restore ()
+static void
+Restore (void)
 {
     Message (message, "Restore not implemented");
 }
 
-Save ()
+static void
+Save (void)
 {
     Message (message, "Save not implemented");
 }
@@ -701,57 +702,67 @@ Save ()
 /* Callbacks to user interface functions */
 
 static void
-CardsCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure;
-    XtPointer	data;
+InputCallback (Widget w, XtPointer closure, XtPointer data)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
-    CardPtr	    card;
-    String	    type;
-    int		    i;
-    Boolean	    hintForward;
+    HandInputPtr    input = (HandInputPtr) data;
+    CardStackPtr    stack = NULL;
+    CardStackPtr    startStack = NULL;
+    int             i;
+    Boolean         hintForward = True;
+    String          type = "";
+
+    (void) closure;
+    (void) w;
+    Message (message, "");
+
+    if (*input->num_params > 0)
+        type = input->params[0];
+
+    stack = &cardStacks[input->current.row * NUM_COLS + input->current.col];
+    startStack = &cardStacks[input->start.row * NUM_COLS + input->start.col];
+
 #define MOVE	0
 #define HINT	1
 #define UNHINT	2
 #define	SELECT	3
 
-    Message (message, "");
-    stack = &cardStacks[input->row * NUM_COLS + input->col];
-    if (!*input->num_params)
-	return;
-    type = *input->params;
-    if (!strcmp (type, "hint-or-select"))
-    {
-	if (!stack->last)
+    switch (input->action) {
+    case HandActionStart:
+	if (!stack->last) {
 	    i = HINT;
-	else
+            hintForward = True;
+	} else
 	    i = SELECT;
-    } else if (!strcmp (type, "unhint-or-move")) {
+        break;
+    case HandActionDrag:
+        CardSetAnimate(False);
+        i = MOVE;
+        break;
+    case HandActionClick:
+        CardSetAnimate(True);
 	if (hintStack)
 	    i = UNHINT;
 	else
 	    i = MOVE;
-    } else if (!strcmp (type, "hint-previous")) {
-	i = HINT;
-	hintForward = False;
-    } else if (!strcmp (type, "hint-next")) {
-	i = HINT;
-	hintForward = True;
-    } else if (!strcmp (type, "unhint")) {
-	i = UNHINT;
-    } else if (!strcmp (type, "hint")) {
-	i = HINT;
-	hintForward = True;
-    } else {
-	return;
+        break;
+    case HandActionExpand:
+        i = HINT;
+        if (!strcmp(type, "hint-previous"))
+            hintForward = False;
+        else
+            hintForward = True;
+        break;
+    default:
+    case HandActionUnexpand:
+        i = UNHINT;
+        break;
     }
+
     switch (i) {
     case HINT:
 	if (!stack->last)
 	{
-	    if (input->col == 0)
+	    if (input->current.col == 0)
 		return;
 	    stack = stack - 1;
 	    hintForward = False;
@@ -762,186 +773,195 @@ CardsCallback (w, closure, data)
 	UnHint ();
 	break;
     case SELECT:
-	fromStack = stack;
 	break;
     case MOVE:
-	if (fromStack) {
-	    Play (fromStack, stack);
-	    fromStack = NULL;
-	}
+	if (startStack)
+	    Play (startStack, stack);
 	break;
     }
     DisplayStacks ();
 }
 
 static void
-DealCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+DealCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Deal ();
 }
 
 static void
-NewGameCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+NewGameCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     NewGame ();
 }
 
 static void
-QuitCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+QuitCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Quit ();
 }
 
 static void
-ScoreCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+ScoreCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Score ();
 }
 
 static void
-UndoCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+UndoCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Undo ();
 }
 
 static void
-RestoreCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+HintCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
+    BestMove ();
+}
+
+static void
+RestoreCallback (Widget w, XtPointer closure, XtPointer data)
+{
+    (void) w;
+    (void) closure;
+    (void) data;
     Restore ();
 }
 
 static void
-SaveCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+SaveCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Save ();
 }
 
 /* actions to user interface functions */
 
-static void UndoAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void UndoAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Undo ();
 }
-    
-static void DealAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void DealAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
-    Deal ();
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
+    Undo ();
 }
-    
-static void NewGameAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void NewGameAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     NewGame ();
 }
-    
-static void ScoreAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void ScoreAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Score ();
 }
-    
-static void QuitAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void QuitAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Quit ();
 }
 
-static void RestoreAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void RestoreAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Restore ();
 }
 
-static void SaveAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void SaveAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Save ();
 }
 
-static void BestMoveAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void BestMoveAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     BestMove ();
 }
 
 XtActionsRec	actions[] = {
-    "montanaUndo",	UndoAction,
-    "montanaDeal",	DealAction,
-    "montanaNewGame",	NewGameAction,
-    "montanaScore",	ScoreAction,
-    "montanaQuit",	QuitAction,
-    "montanaRestore",	RestoreAction,
-    "montanaSave",	SaveAction,
-    "montanaBestMove",	BestMoveAction,
+    { "montanaUndo",	UndoAction, },
+    { "montanaDeal",	DealAction, },
+    { "montanaNewGame",	NewGameAction, },
+    { "montanaScore",	ScoreAction, },
+    { "montanaQuit",	QuitAction, },
+    { "montanaRestore",	RestoreAction, },
+    { "montanaSave",	SaveAction, },
+    { "montanaBestMove",	BestMoveAction, },
 };
 
 struct menuEntry {
     char    *name;
-    void    (*function)();
+    void    (*function)(Widget w, XtPointer closure, XtPointer data);
 };
 
 struct menuEntry fileMenuEntries[] = {
-    "restore", RestoreCallback,
-    "save", SaveCallback,
-    "quit", QuitCallback,
+    { "restore", RestoreCallback, },
+    { "save", SaveCallback, },
+    { "quit", QuitCallback, },
 };
-    
-Widget
-CreateMenu (parent, name, entries, count)
-    Widget  parent;
-    char    *name;
-    struct menuEntry	*entries;
-    int	    count;
+
+static Widget
+CreateMenu (Widget parent, char *name, struct menuEntry *entries, int count)
 {
     Widget  menu;
     Widget  entry;
     int	    i;
 
-    menu = XtCreatePopupShell (name, simpleMenuWidgetClass,
+    menu = XtCreatePopupShell (name, ksimpleMenuWidgetClass,
 			       parent, NULL, ZERO);
     for (i = 0; i < count; i++) {
 	entry = XtCreateManagedWidget (entries[i].name,
-				       smeBSBObjectClass, menu, NULL, ZERO);
+				       ksmeBSBObjectClass, menu, NULL, ZERO);
 	XtAddCallback (entry, XtNcallback, entries[i].function, NULL);
     }
     return menu;
@@ -953,61 +973,60 @@ XtResource resources[] = {
     { "animationSpeed", "AnimationSpeed", XtRInt, sizeof (int),
      offset(animationSpeed), XtRImmediate, (XtPointer) -1},
     { "searchDepth", "SearchDepth", XtRInt, sizeof (int),
-     offset(searchDepth), XtRImmediate, (XtPointer) MAX_SEARCH},
+     offset(searchDepth), XtRImmediate, (XtPointer) 40},
 };
 
 XrmOptionDescRec options[] = {
-    "-smallCards",	"*Cards.smallCards",	XrmoptionNoArg, "True",
-    "-squareCards",	"*Cards.roundCards",	XrmoptionNoArg, "False",
-    "-noanimate",	"*animationSpeed",	XrmoptionNoArg, "0",
-    "-animationSpeed",	"*animationSpeed",	XrmoptionSepArg, NULL,
-    "-search",		"*searchDepth",		XrmoptionSepArg, NULL,
+    { "-noanimate",	"*animationSpeed",	XrmoptionNoArg, "0", },
+    { "-animationSpeed","*animationSpeed",	XrmoptionSepArg, NULL },
+    { "-searchDepth",   "*searchDepth",         XrmoptionSepArg, NULL },
 };
 
-main (argc, argv)
-    int	    argc;
-    char    **argv;
+int
+main (int argc, char **argv)
 {
-    Arg	args[20];
     Atom wm_delete_window;
 
-    toplevel = XtInitialize (argv[0], "KMontana", options, XtNumber(options),
-			     &argc, argv);
-    
+    toplevel = XkwInitialize ("KMontana", options, XtNumber(options),
+			      &argc, argv, True, defaultResources);
+
     XtGetApplicationResources (toplevel, (XtPointer)&montanaResources, resources,
 			       XtNumber (resources), NULL, 0);
-    
+
     AnimateSetSpeed (montanaResources.animationSpeed);
-    
+
     XtAddActions (actions, XtNumber(actions));
 
-    XtOverrideTranslations 
-	(toplevel, 
+    XtOverrideTranslations
+	(toplevel,
 	 XtParseTranslationTable ("<Message>WM_PROTOCOLS: montanaQuit()"));
     frame = XtCreateManagedWidget ("frame", layoutWidgetClass, toplevel, NULL, 0);
     menuBar = XtCreateManagedWidget ("menuBar", layoutWidgetClass, frame, NULL, 0);
     fileMenuButton = XtCreateManagedWidget ("fileMenuButton",
-					    menuButtonWidgetClass,
+					    kmenuButtonWidgetClass,
 					    menuBar, NULL, ZERO);
-    fileMenu = CreateMenu (fileMenuButton, "fileMenu", 
+    fileMenu = CreateMenu (fileMenuButton, "fileMenu",
 			   fileMenuEntries, XtNumber (fileMenuEntries));
-    deal = XtCreateManagedWidget ("deal", commandWidgetClass,
+    deal = XtCreateManagedWidget ("deal", kcommandWidgetClass,
 				     menuBar, NULL, ZERO);
     XtAddCallback(deal, XtNcallback, DealCallback, NULL);
-    dealDisplay = XtCreateManagedWidget ("dealDisplay", labelWidgetClass,
+    dealDisplay = XtCreateManagedWidget ("dealDisplay", klabelWidgetClass,
 					frame, NULL, ZERO);
-    newGame = XtCreateManagedWidget ("newGame", commandWidgetClass,
+    newGame = XtCreateManagedWidget ("newGame", kcommandWidgetClass,
 				     menuBar, NULL, ZERO);
     XtAddCallback(newGame, XtNcallback, NewGameCallback, NULL);
-    undo = XtCreateManagedWidget ("undo", commandWidgetClass,
+    undo = XtCreateManagedWidget ("undo", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(undo, XtNcallback, UndoCallback, NULL);
-    score = XtCreateManagedWidget ("score", commandWidgetClass,
+    hint = XtCreateManagedWidget ("hint", kcommandWidgetClass,
+				  menuBar, NULL, ZERO);
+    XtAddCallback(hint, XtNcallback, HintCallback, NULL);
+    score = XtCreateManagedWidget ("score", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(score, XtNcallback, ScoreCallback, NULL);
     cards = XtCreateManagedWidget ("cards", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (cards, XtNinputCallback, CardsCallback, NULL);
-    message = XtCreateManagedWidget ("message", labelWidgetClass, frame, NULL, 0);
+    XtAddCallback (cards, XtNinputCallback, InputCallback, NULL);
+    message = XtCreateManagedWidget ("message", klabelWidgetClass, frame, NULL, 0);
     srandom (getpid () ^ time ((long *) 0));
     NewGame ();
     XtRealizeWidget (toplevel);
@@ -1016,5 +1035,7 @@ main (argc, argv)
     (void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
                             &wm_delete_window, 1);
 
+    XkwSetCardIcon(toplevel);
+
     XtMainLoop ();
 }
