@@ -17,34 +17,32 @@
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NCD.
  * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
+ * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  * Author:  Keith Packard, Network Computing Devices
  */
 
+# include	<stdlib.h>
 # include	<X11/Intrinsic.h>
 # include	<X11/StringDefs.h>
 # include	<X11/Shell.h>
 # include	<X11/Xos.h>
-# include	<X11/Xaw/Command.h>
+# include	<Xkw/KCommand.h>
 # include	<X11/Xaw/Box.h>
 # include	<X11/Xaw/Dialog.h>
-# include	<X11/Xaw/Label.h>
-# include	<X11/Xaw/MenuButton.h>
-# include	<X11/Xaw/SimpleMenu.h>
-# include	<X11/Xaw/SmeBSB.h>
+# include	<Xkw/KLabel.h>
+# include	<Xkw/KMenuButton.h>
+# include	<Xkw/KSimpleMenu.h>
+# include	<Xkw/KSmeBSB.h>
 # include	<X11/Xaw/AsciiText.h>
 # include	<X11/Xaw/Cardinals.h>
 # include	<Xkw/Cards.h>
 # include	<Xkw/Layout.h>
 # include	<X11/Xutil.h>
 # include	<Xkw/CardsUtil.h>
-
-#if defined(hpux) || defined(__hpux)
-#define srandom srand48
-#define random lrand48
-#endif
+# include	<Xkw/Message.h>
+# include	"KThieves-res.h"
 
 Widget	    toplevel;
 Widget	    frame;
@@ -83,7 +81,7 @@ typedef struct _thievesResources {
     int		animationSpeed;
     Boolean    	squishCards;
     int		numColumns;
-    int		numRows
+    int		numRows;
 } ThievesResources, *ThievesResourcesPtr;
 
 ThievesResources thievesResources;
@@ -91,12 +89,13 @@ ThievesResources thievesResources;
 #define numStacks   thievesResources.numColumns
 #define firstRows   thievesResources.numRows
 
-InitStacks ()
+static void
+InitStacks (void)
 {
     int	    col;
     int	    row;
     CardDisplay	    display;
-    
+
     for (col = 0; col < numStacks; col++)
     {
 	display = CardDisplayAll;
@@ -118,13 +117,14 @@ InitStacks ()
 		   draw, CardsNone, True, 0, CardDisplayAll);
 }
 
-GenerateCards ()
+static void
+GenerateCards (void)
 {
     int		    i;
     CardPtr	    card;
 
     card = rawcards;
-    for (i = 0; i < 2; i++) 
+    for (i = 0; i < 2; i++)
     {
 	CardGenerateStandardDeck (card);
 	card += 52;
@@ -135,13 +135,14 @@ GenerateCards ()
     deckStack.last = &rawcards[NUM_CARDS-1];
 }
 
-FirstDeal ()
+static void
+FirstDeal (void)
 {
     int	    row, col;
-    
-    for (row = 0; row < firstRows; row++) 
+
+    for (row = 0; row < firstRows; row++)
     {
-	for (col = 0; col < numStacks; col++) 
+	for (col = 0; col < numStacks; col++)
 	{
 	    CardMove (&deckStack, deckStack.last, &stackStacks[col], False);
 	    CardTurn (stackStacks[col].last, CardFaceUp, False);
@@ -151,16 +152,9 @@ FirstDeal ()
     Message (deckCount, "%d", dealNumber);
 }
 
-CheckStackTop (stack)
-    CardStackPtr    stack;
-{
-    if (stack->last && stack->last->face == CardFaceDown)
-	CardTurn (stack->last, CardFaceUp, True);
-}
-
 /*
  * Score:
- 
+
  each card played on the right counts as its face value
  ace ==> 1
  duece ==> 2
@@ -175,14 +169,13 @@ a winning game has 728 points.
 
  */
 
-int
-ComputeScore ()
+static int
+ComputeScore (void)
 {
     int		    score = 0;
     int		    col;
     CardStackPtr    pile;
     CardPtr	    card;
-    int		    numPiles = 0;
 
     for (col = 0; col < NUM_PILES; col++)
     {
@@ -190,22 +183,23 @@ ComputeScore ()
 	for (card = pile->first; card; card = card->next)
 	    score += CardsRankToInt (card->card.rank) + 1;
     }
-    
+
     return (score);
 }
-    
-DisplayStacks ()
+
+static void
+DisplayStacks (void)
 {
     int		    col;
     CardPtr	    card, c;
     CardStackPtr    stack;
-    
+
     CardDisplayStack (&deckStack);
     CardDisplayStack (&drawStack);
-    
+
     for (col = 0; col < NUM_PILES; col++)
 	CardDisplayStack (&pileStacks[col]);
-    
+
     for (col = 0; col < numStacks; col++)
     {
 	stack = &stackStacks[col];
@@ -245,32 +239,29 @@ DisplayStacks ()
 
 /* User interface functions */
 
-void
-ResetDealNumber (closure)
-    char    *closure;
+static void
+ResetDealNumber (void *closure)
 {
-    dealNumber = (int) closure;
+    dealNumber = (int) (intptr_t) closure;
     Message (deckCount, "%d", dealNumber);
 }
 
-void
-Deal ()
+static void
+Deal (void)
 {
-    CardPtr   c;
-
-    if (!deckStack.last) 
+    if (!deckStack.last)
     {
 	Message (message, "No more cards in the deck.");
 	return;
     }
     CardMove (&deckStack, deckStack.last, &drawStack, True);
     CardTurn (drawStack.last, CardFaceUp, True);
-    CardRecordHistoryCallback (ResetDealNumber, (char *) dealNumber);
+    CardRecordHistoryCallback (ResetDealNumber, (char *) (intptr_t) dealNumber);
     Message (deckCount, "%d", --dealNumber);
 }
 
-void
-NewGame ()
+static void
+NewGame (void)
 {
     CardsRemoveAllCards (deck);
     CardsRemoveAllCards (piles);
@@ -280,46 +271,46 @@ NewGame ()
     fromCard = 0;
     InitStacks ();
     GenerateCards ();
-    CardShuffle (&deckStack);
+    CardShuffle (&deckStack, False);
     FirstDeal ();
     CardInitHistory ();
     DisplayStacks ();
 }
 
-void
-Undo ()
+static void
+Undo (void)
 {
     if (!CardUndo ())
 	Message (message, "Nothing to undo.");
     DisplayStacks ();
 }
 
-void
-Score ()
+#define MAX_SCORE 728
+
+static void
+Score (void)
 {
-    Message (message, "Current position scores %d out of 728.", 
-	     ComputeScore ());
+    Message (message, "Current position scores %d out of %d.",
+	     ComputeScore (), MAX_SCORE);
 }
 
-void
-Quit ()
+static void
+Quit (void)
 {
     exit (0);
 }
 
-CardStackPtr
-FindFinishPlay (from_card)
-    CardPtr	    from_card;
+static CardStackPtr
+FindFinishPlay (CardPtr from_card)
 {
     int		    i;
     CardStackPtr    to_stack;
-    CardPtr	    card;
-    
+
     for (i = 0; i < NUM_PILES; i++)
     {
 	to_stack = &pileStacks[i];
-	if (from_card->card.rank == CardsAce && to_stack->last == NULL ||
-	    to_stack->last && CardIsInSuitOrder (to_stack->last, from_card))
+	if ((from_card->card.rank == CardsAce && to_stack->last == NULL) ||
+	    (to_stack->last && CardIsInSuitOrder (to_stack->last, from_card)))
 	{
 	    return to_stack;
 	}
@@ -327,31 +318,30 @@ FindFinishPlay (from_card)
     return NULL;
 }
 
-CardStackPtr
-FindStackPlay (from_card)
-    CardPtr	    from_card;
+static CardStackPtr
+FindStackPlay (CardPtr from_card)
 {
     int		    i;
     CardStackPtr    to_stack;
     CardPtr	    card;
-    
+
     for (i = 0; i < numStacks; i++)
     {
 	to_stack = &stackStacks[i];
 	card = to_stack->last;
-	if (card && CardIsInSuitOrder (from_card, card))
+	if (card && CardIsInOrder (from_card, card))
 	    return to_stack;
     }
     return NULL;
 }
 
-CardStackPtr
-FindEmptyPlay (from_card)
-    CardPtr	    from_card;
+static CardStackPtr
+FindEmptyPlay (CardPtr from_card)
 {
     int		    i;
     CardStackPtr    to_stack;
-    
+
+    (void) from_card;
     for (i = 0; i < numStacks; i++)
     {
 	to_stack = &stackStacks[i];
@@ -361,15 +351,9 @@ FindEmptyPlay (from_card)
     return NULL;
 }
 
-void
-Play (from_stack, from_card, to_stack)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
-    CardStackPtr    to_stack;
+static void
+Play (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
-    int		    i;
-    CardPtr	    card;
-
     if (to_stack != from_stack)
     {
 	if (to_stack->last != NULL && !
@@ -389,16 +373,19 @@ Play (from_stack, from_card, to_stack)
 	}
     }
     CardMove (from_stack, from_card, to_stack, True);
+    if (ComputeScore() == MAX_SCORE)
+        Message(message, "We have a winner!");
 }
 
 static Boolean
-AlreadyEmpty (a,b)
-    CardPtr a, b;
+AlreadyEmpty (CardPtr a, CardPtr b)
 {
+    (void) a;
     return !b;
 }
 
-FindAMove ()
+static void
+FindAMove (void)
 {
     int		    col;
     CardStackPtr    from_stack, to_stack;
@@ -408,9 +395,9 @@ FindAMove ()
     to_stack = NULL;
     for (col = 0; col < numStacks; col++)
 	goodenough[col] = False;
-    
+
 #define FindOneInStack(from_stack,func) \
-    if (from_card = (from_stack)->last) \
+    if ((from_card = (from_stack)->last))	\
 	to_stack = func(from_card);
 
 #define FindOneCheck(already, func) \
@@ -427,7 +414,7 @@ FindAMove ()
     if (!to_stack) { \
 	FindOneInStack (&drawStack,func); \
     }
-    
+
 #define FindOne(func) \
     for (col = 0; !to_stack && col < numStacks; col++) {\
 	from_stack = &stackStacks[col]; \
@@ -461,22 +448,24 @@ FindAMove ()
     }
 }
 
-Restore ()
+static void
+Restore (void)
 {
     Message (message, "Restore not implemented");
 }
 
-Save ()
+static void
+Save (void)
 {
     Message (message, "Save not implemented");
 }
 
-Expand (stack)
-    CardStackPtr    stack;
+static void
+Expand (CardStackPtr stack)
 {
     CardPtr card, t;
 
-    if (card = stack->first) {
+    if ((card = stack->first)) {
 	MessageStart ();
 	MessageAppend ("Column contains:");
 	while (card) {
@@ -501,225 +490,234 @@ Expand (stack)
 
 /* Callbacks to user interface functions */
 
-static void
-DeckCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure;
-    XtPointer	data;
+static CardStackPtr
+WidgetToStack(Widget w, int col)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
-    CardPtr	    card;
-    
-    Message (message, "");
-    Deal ();
-    CardNextHistory ();
-    DisplayStacks ();
+    if (w == stacks)
+	return &stackStacks[col];
+    if (w == piles)
+        return &pileStacks[col];
+    if (w == draw)
+	return &drawStack;
+    if (w == deck)
+        return &deckStack;
+    return NULL;
 }
 
 static void
-StackCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure;
-    XtPointer	data;
+InputCallback (Widget w, XtPointer closure, XtPointer data)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
-    CardPtr	    card;
-    String	    type;
-    int		    i;
+    HandInputPtr    input = (HandInputPtr) data;
+    CardStackPtr    stack = NULL;
+    CardStackPtr    startStack = NULL;
+    CardPtr	    card = NULL;
 
+    (void) closure;
     Message (message, "");
-    if (w == stacks)
-	stack = &stackStacks[input->col];
-    else if (w == draw)
-	stack = &drawStack;
-    card = stack->last;
-    if (*input->num_params) {
-	type = *input->params;
-	if (!strcmp (type, "source"))
-	{
-	    fromStack = stack;
-	    if (fromStack->last)
-		fromCard = fromStack->last;
-	    else
-		Message (message, "Selected stack is empty.");
-	}
-	else if (!strcmp (type, "dest"))
-	{
-	    if (fromCard)
-	    {
-		Play (fromStack, fromCard, stack);
-		fromCard = NULL;
+    stack = WidgetToStack(w, input->current.col);
+    startStack = WidgetToStack(input->start.w, input->start.col);
+
+    if (!startStack || !stack)
+	return;
+
+    CardSetAnimate(True);
+    switch (input->action) {
+    case HandActionStart:
+	break;
+    case HandActionDrag:
+        if (startStack == stack)
+            break;
+        CardSetAnimate(False);
+        /* fall through */
+    case HandActionClick:
+	if (startStack == &deckStack) {
+	    if (stack == &deckStack) {
+		Deal ();
 		CardNextHistory ();
 		DisplayStacks ();
 	    }
 	}
-	else if (!strcmp (type, "expand"))
-	{
-	    Expand (stack);
+	else if (stack == &deckStack) {
 	}
+        else {
+            card = startStack->last;
+            if (card)
+            {
+                Play (startStack, card, stack);
+                CardNextHistory ();
+                DisplayStacks ();
+            }
+        }
+        break;
+    case HandActionExpand:
+        Expand (stack);
+        break;
+    case HandActionUnexpand:
+        break;
     }
 }
 
 static void
-NewGameCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+NewGameCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     NewGame ();
 }
 
 static void
-QuitCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+QuitCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Quit ();
 }
 
 static void
-ScoreCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+ScoreCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Score ();
 }
 
 static void
-UndoCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+UndoCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Undo ();
 }
 
 static void
-FindAMoveCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+FindAMoveCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     FindAMove ();
 }
 
 static void
-RestoreCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+RestoreCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Restore ();
 }
 
 static void
-SaveCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+SaveCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Save ();
 }
 
 /* actions to user interface functions */
 
-static void UndoAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void UndoAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Undo ();
 }
-    
-static void NewGameAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void NewGameAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     NewGame ();
 }
-    
-static void ScoreAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void ScoreAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Score ();
 }
-    
-static void QuitAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void QuitAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Quit ();
 }
 
-static void FindAMoveAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void FindAMoveAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     FindAMove ();
 }
 
-static void RestoreAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void RestoreAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Restore ();
 }
 
-static void SaveAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void SaveAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Save ();
 }
 
 XtActionsRec	actions[] = {
-    "thievesUndo",	UndoAction,
-    "thievesNewGame",	NewGameAction,
-    "thievesScore",	ScoreAction,
-    "thievesQuit",	QuitAction,
-    "thievesFindAMove",	FindAMoveAction,
-    "thievesRestore",	RestoreAction,
-    "thievesSave",	SaveAction,
+    { "thievesUndo",	UndoAction, },
+    { "thievesNewGame",	NewGameAction, },
+    { "thievesScore",	ScoreAction, },
+    { "thievesQuit",	QuitAction, },
+    { "thievesFindAMove",	FindAMoveAction, },
+    { "thievesRestore",	RestoreAction, },
+    { "thievesSave",	SaveAction, },
 };
 
 struct menuEntry {
     char    *name;
-    void    (*function)();
+    void    (*function)(Widget w, XtPointer closure, XtPointer data);
 };
 
 struct menuEntry fileMenuEntries[] = {
-    "restore", RestoreCallback,
-    "save", SaveCallback,
-    "quit", QuitCallback,
+    { "restore", RestoreCallback, },
+    { "save", SaveCallback, },
+    { "quit", QuitCallback, },
 };
-    
-Widget
-CreateMenu (parent, name, entries, count)
-    Widget  parent;
-    char    *name;
-    struct menuEntry	*entries;
-    int	    count;
+
+static Widget
+CreateMenu (Widget parent, char *name, struct menuEntry *entries, int count)
 {
     Widget  menu;
     Widget  entry;
     int	    i;
 
-    menu = XtCreatePopupShell (name, simpleMenuWidgetClass,
+    menu = XtCreatePopupShell (name, ksimpleMenuWidgetClass,
 			       parent, NULL, ZERO);
     for (i = 0; i < count; i++) {
 	entry = XtCreateManagedWidget (entries[i].name,
-				       smeBSBObjectClass, menu, NULL, ZERO);
+				       ksmeBSBObjectClass, menu, NULL, ZERO);
 	XtAddCallback (entry, XtNcallback, entries[i].function, NULL);
     }
     return menu;
@@ -739,66 +737,65 @@ XtResource resources[] = {
 };
 
 XrmOptionDescRec options[] = {
-    "-squish",		"*squishCards",		XrmoptionNoArg,	"True",
-    "-squareCards",	"*Cards.roundCards",	XrmoptionNoArg, "False",
-    "-smallCards",	"*Cards.smallCards",	XrmoptionNoArg, "True",
-    "-noanimate",	"*animationSpeed",	XrmoptionNoArg, "0",
-    "-animationSpeed",	"*animationSpeed",	XrmoptionSepArg, NULL,
-    "-cols",		".numColumns",		XrmoptionSepArg, NULL,
-    "-rows",		".numRows",		XrmoptionSepArg, NULL,
+    { "-squish",		"*squishCards",		XrmoptionNoArg,	"True", },
+    { "-squareCards",	"*Cards.roundCards",	XrmoptionNoArg, "False", },
+    { "-smallCards",	"*Cards.smallCards",	XrmoptionNoArg, "True", },
+    { "-noanimate",	"*animationSpeed",	XrmoptionNoArg, "0", },
+    { "-animationSpeed",	"*animationSpeed",	XrmoptionSepArg, NULL, },
+    { "-cols",		".numColumns",		XrmoptionSepArg, NULL, },
+    { "-rows",		".numRows",		XrmoptionSepArg, NULL, },
 };
 
-main (argc, argv)
-    int	    argc;
-    char    **argv;
+int
+main (int argc, char **argv)
 {
     Arg	args[20];
     int	nargs;
     Atom wm_delete_window;
 
-    toplevel = XtInitialize (argv[0], "KThieves", options, XtNumber(options),
-			     &argc, argv);
-    
+    toplevel = XkwInitialize ("KThieves", options, XtNumber(options),
+			      &argc, argv, True, defaultResources);
+
     XtGetApplicationResources (toplevel, (XtPointer)&thievesResources, resources,
 			       XtNumber (resources), NULL, 0);
-    
+
     AnimateSetSpeed (thievesResources.animationSpeed);
-    
+
     XtAddActions (actions, XtNumber(actions));
 
-    XtOverrideTranslations 
-	(toplevel, 
+    XtOverrideTranslations
+	(toplevel,
 	 XtParseTranslationTable ("<Message>WM_PROTOCOLS: thievesQuit()"));
     frame = XtCreateManagedWidget ("frame", layoutWidgetClass, toplevel, NULL, 0);
     menuBar = XtCreateManagedWidget ("menuBar", layoutWidgetClass, frame, NULL, 0);
     fileMenuButton = XtCreateManagedWidget ("fileMenuButton",
-					    menuButtonWidgetClass,
+					    kmenuButtonWidgetClass,
 					    menuBar, NULL, ZERO);
-    fileMenu = CreateMenu (fileMenuButton, "fileMenu", 
+    fileMenu = CreateMenu (fileMenuButton, "fileMenu",
 			   fileMenuEntries, XtNumber (fileMenuEntries));
-    newGame = XtCreateManagedWidget ("newGame", commandWidgetClass,
+    newGame = XtCreateManagedWidget ("newGame", kcommandWidgetClass,
 				     menuBar, NULL, ZERO);
     XtAddCallback(newGame, XtNcallback, NewGameCallback, NULL);
-    undo = XtCreateManagedWidget ("undo", commandWidgetClass,
+    undo = XtCreateManagedWidget ("undo", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(undo, XtNcallback, UndoCallback, NULL);
-    hint = XtCreateManagedWidget ("hint", commandWidgetClass,
+    hint = XtCreateManagedWidget ("hint", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(hint, XtNcallback, FindAMoveCallback, NULL);
-    score = XtCreateManagedWidget ("score", commandWidgetClass,
+    score = XtCreateManagedWidget ("score", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(score, XtNcallback, ScoreCallback, NULL);
     deck = XtCreateManagedWidget ("deck", cardsWidgetClass, frame, NULL, 0);
-    deckCount = XtCreateManagedWidget ("deckCount", labelWidgetClass, frame, NULL, 0);
-    XtAddCallback (deck, XtNinputCallback, DeckCallback, NULL);
+    deckCount = XtCreateManagedWidget ("deckCount", klabelWidgetClass, frame, NULL, 0);
+    XtAddCallback (deck, XtNinputCallback, InputCallback, NULL);
     draw = XtCreateManagedWidget ("draw", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (draw, XtNinputCallback, StackCallback, NULL);
+    XtAddCallback (draw, XtNinputCallback, InputCallback, NULL);
     piles = XtCreateManagedWidget ("piles", cardsWidgetClass, frame, NULL, 0);
     nargs = 0;
     XtSetArg (args[nargs], XtNnumCols, numStacks); nargs++;
     stacks = XtCreateManagedWidget ("stacks", cardsWidgetClass, frame, args, nargs);
-    XtAddCallback (stacks, XtNinputCallback, StackCallback, NULL);
-    message = XtCreateManagedWidget ("message", labelWidgetClass, frame, NULL, 0);
+    XtAddCallback (stacks, XtNinputCallback, InputCallback, NULL);
+    message = XtCreateManagedWidget ("message", klabelWidgetClass, frame, NULL, 0);
     srandom (getpid () ^ time ((long *) 0));
     NewGame ();
     XtRealizeWidget (toplevel);
@@ -807,5 +804,7 @@ main (argc, argv)
     (void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
                             &wm_delete_window, 1);
 
+    XkwSetCardIcon(toplevel);
+
     XtMainLoop ();
 }
