@@ -17,34 +17,32 @@
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NCD.
  * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
+ * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  * Author:  Keith Packard, Network Computing Devices
  */
 
+# include	<stdlib.h>
 # include	<X11/Intrinsic.h>
 # include	<X11/StringDefs.h>
 # include	<X11/Shell.h>
 # include	<X11/Xos.h>
-# include	<X11/Xaw/Command.h>
+# include	<Xkw/KCommand.h>
 # include	<X11/Xaw/Box.h>
 # include	<X11/Xaw/Dialog.h>
-# include	<X11/Xaw/Label.h>
-# include	<X11/Xaw/MenuButton.h>
-# include	<X11/Xaw/SimpleMenu.h>
-# include	<X11/Xaw/SmeBSB.h>
+# include	<Xkw/KLabel.h>
+# include	<Xkw/KMenuButton.h>
+# include	<Xkw/KSimpleMenu.h>
+# include	<Xkw/KSmeBSB.h>
 # include	<X11/Xaw/AsciiText.h>
 # include	<X11/Xaw/Cardinals.h>
 # include	<Xkw/Cards.h>
 # include	<Xkw/Layout.h>
 # include	<X11/Xutil.h>
 # include	<Xkw/CardsUtil.h>
-
-#if defined(hpux) || defined(__hpux)
-#define srandom srand48
-#define random lrand48
-#endif
+# include	<Xkw/Message.h>
+# include	"KTowers-res.h"
 
 Widget	    toplevel;
 Widget	    frame;
@@ -81,11 +79,11 @@ typedef struct _ktowersResources {
 
 KtowersResources ktowersResources;
 
-InitStacks ()
+static void
+InitStacks (void)
 {
     int		    col;
-    CardDisplay	    display;
-    
+
     for (col = 0; col < NUM_STACKS; col++)
     {
 	CardInitStack (&stackStacks[col],
@@ -103,7 +101,8 @@ InitStacks ()
     }
 }
 
-GenerateCards ()
+static void
+GenerateCards (void)
 {
     CardGenerateStandardDeck (rawcards);
     suitStacks[0].first = &rawcards[0];
@@ -112,13 +111,14 @@ GenerateCards ()
 
 #define FIRST_ROWS  5
 
-FirstDeal ()
+static void
+FirstDeal (void)
 {
     int	    row, col;
-    
-    for (row = 0; row < FIRST_ROWS; row++) 
+
+    for (row = 0; row < FIRST_ROWS; row++)
     {
-	for (col = 0; col < NUM_STACKS; col++) 
+	for (col = 0; col < NUM_STACKS; col++)
 	{
 	    CardMove (&suitStacks[0], suitStacks[0].last, &stackStacks[col], False);
 	    CardTurn (stackStacks[col].last, CardFaceUp, False);
@@ -133,8 +133,8 @@ FirstDeal ()
     }
 }
 
-int
-ComputeScore ()
+static int
+ComputeScore (void)
 {
     int	    col;
     int	    score;
@@ -153,17 +153,18 @@ ComputeScore ()
     }
     return score;
 }
-    
-DisplayStacks ()
+
+static void
+DisplayStacks (void)
 {
     int		    col;
-    
+
     for (col = 0; col < NUM_SUITS; col++)
 	CardDisplayStack (&suitStacks[col]);
-    
+
     for (col = 0; col < NUM_PILES; col++)
 	CardDisplayStack (&pileStacks[col]);
-    
+
     for (col = 0; col < NUM_STACKS; col++)
 	CardDisplayStack (&stackStacks[col]);
     CardsUpdateDisplay (suits);
@@ -173,8 +174,8 @@ DisplayStacks ()
 
 /* User interface functions */
 
-void
-NewGame ()
+static void
+NewGame (void)
 {
     CardsRemoveAllCards (suits);
     CardsRemoveAllCards (piles);
@@ -183,45 +184,45 @@ NewGame ()
     fromCard = 0;
     InitStacks ();
     GenerateCards ();
-    CardShuffle (&suitStacks[0]);
+    CardShuffle (&suitStacks[0], False);
     FirstDeal ();
     CardInitHistory ();
     DisplayStacks ();
 }
 
-void
-Undo ()
+static void
+Undo (void)
 {
     if (!CardUndo ())
 	Message (message, "Nothing to undo.");
     DisplayStacks ();
 }
 
-void
-Score ()
+#define MAX_SCORE 340
+
+static void
+Score (void)
 {
-    Message (message, "Current position scores %d out of 340.", 
-	     ComputeScore ());
+    Message (message, "Current position scores %d out of %d.",
+	     ComputeScore (), MAX_SCORE);
 }
 
-void
-Quit ()
+static void
+Quit (void)
 {
     exit (0);
 }
 
-Boolean
-IsLegalSuitPlay (from_stack, from_card, to_stack)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
-    CardStackPtr    to_stack;
+static Boolean
+IsLegalSuitPlay (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
+    (void) from_stack;
     if (from_card->card.rank == CardsAce)
     {
 	if (to_stack->last == NULL)
 	    return True;
-    } 
-    else 
+    }
+    else
     {
 	if (to_stack->last != NULL &&
 	    CardIsInSuitOrder (to_stack->last, from_card))
@@ -230,14 +231,12 @@ IsLegalSuitPlay (from_stack, from_card, to_stack)
     return False;
 }
 
-CardStackPtr
-FindSuitPlay (from_stack, from_card)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
+static CardStackPtr
+FindSuitPlay (CardStackPtr from_stack, CardPtr from_card)
 {
     int		    i;
     CardStackPtr    to_stack;
-    
+
     for (i = 0; i < NUM_SUITS; i++)
     {
 	to_stack = &suitStacks[i];
@@ -247,25 +246,22 @@ FindSuitPlay (from_stack, from_card)
     return NULL;
 }
 
-Boolean
-IsLegalPilePlay (from_stack, from_card, to_stack)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
-    CardStackPtr    to_stack;
+static Boolean
+IsLegalPilePlay (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
+    (void) from_stack;
+    (void) from_card;
     if (to_stack->last == NULL)
 	return True;
     return False;
 }
 
-CardStackPtr
-FindPilePlay (from_stack, from_card)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
+static CardStackPtr
+FindPilePlay (CardStackPtr from_stack, CardPtr from_card)
 {
     int		    i;
     CardStackPtr    to_stack;
-    
+
     for (i = 0; i < NUM_PILES; i++)
     {
 	to_stack = &pileStacks[i];
@@ -275,28 +271,24 @@ FindPilePlay (from_stack, from_card)
     return NULL;
 }
 
-Boolean
-IsLegalRegularPlay (from_stack, from_card, to_stack)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
-    CardStackPtr    to_stack;
+static Boolean
+IsLegalRegularPlay (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
     CardPtr to_card;
 
+    (void) from_stack;
     to_card = to_stack->last;
     if (to_card && CardIsInSuitOrder (from_card, to_card))
 	return True;
     return False;
 }
 
-CardStackPtr
-FindRegularPlay (from_stack, from_card)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
+static CardStackPtr
+FindRegularPlay (CardStackPtr from_stack, CardPtr from_card)
 {
     int		    i;
     CardStackPtr    to_stack;
-    
+
     for (i = 0; i < NUM_STACKS; i++)
     {
 	to_stack = &stackStacks[i];
@@ -306,26 +298,21 @@ FindRegularPlay (from_stack, from_card)
     return NULL;
 }
 
-Boolean
-IsLegalEmptyPlay (from_stack, from_card, to_stack)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
-    CardStackPtr    to_stack;
+static Boolean
+IsLegalEmptyPlay (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
+    (void) from_stack;
     if (from_card->card.rank == CardsKing && to_stack->last == NULL)
 	return True;
     return False;
 }
 
-CardStackPtr
-FindEmptyPlay (from_stack, from_card)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
+static CardStackPtr
+FindEmptyPlay (CardStackPtr from_stack, CardPtr from_card)
 {
     int		    i;
-    int		    col = from_stack - stackStacks;
     CardStackPtr    to_stack;
-    
+
     for (i = 0; i < NUM_STACKS; i++)
     {
 	to_stack = &stackStacks[i];
@@ -335,15 +322,9 @@ FindEmptyPlay (from_stack, from_card)
     return NULL;
 }
 
-void
-Play (from_stack, from_card, to_stack)
-    CardStackPtr    from_stack;
-    CardPtr	    from_card;
-    CardStackPtr    to_stack;
+static void
+Play (CardStackPtr from_stack, CardPtr from_card, CardStackPtr to_stack)
 {
-    int		    i;
-    CardPtr	    card;
-
     if (!from_card)
     {
 	from_card = from_stack->last;
@@ -381,7 +362,7 @@ Play (from_stack, from_card, to_stack)
 			     &from_card->card, &to_stack->last->card);
 		return;
 	    }
-	} 
+	}
 	else if (to_stack->widget == suits)
 	{
 	    if (!IsLegalSuitPlay (from_stack, from_card, to_stack))
@@ -406,32 +387,19 @@ Play (from_stack, from_card, to_stack)
 	}
     }
     CardMove (from_stack, from_card, to_stack, True);
+    if (ComputeScore() == MAX_SCORE)
+        Message(message, "We have a winner!");
 }
 
-static Boolean
-AlreadyEmpty (a,b)
-    CardPtr a, b;
-{
-    return !b;
-}
-
-static Boolean
-AlreadyInOrder (a, b)
-    CardPtr a, b;
-{
-    if (a && b && CardInSuitOrder (a,b))
-	return True;
-    return False;
-}
-
-FindAMove ()
+static void
+FindAMove (void)
 {
     int		    col;
     CardStackPtr    from_stack, to_stack;
     CardPtr	    from_card;
 
     to_stack = NULL;
-    
+
 #define FindOne(func) \
     for (col = 0; !to_stack && col < NUM_STACKS; col++) {\
 	from_stack = &stackStacks[col]; \
@@ -479,17 +447,20 @@ FindAMove ()
     Message (message, "It's all over.");
 }
 
-Restore ()
+static void
+Restore (void)
 {
     Message (message, "Restore not implemented");
 }
 
-Save ()
+static void
+Save (void)
 {
     Message (message, "Save not implemented");
 }
 
-PileAll ()
+static void
+PileAll (void)
 {
     CardStackPtr    from_stack;
     CardPtr	    from_card;
@@ -513,12 +484,12 @@ PileAll ()
 	Message (message, "No cards to pile.");
 }
 
-Expand (stack)
-    CardStackPtr    stack;
+static void
+Expand (CardStackPtr stack)
 {
     CardPtr card, t;
 
-    if (card = stack->first) {
+    if ((card = stack->first)) {
 	MessageStart ();
 	MessageAppend ("Column contains:");
 	while (card) {
@@ -543,232 +514,237 @@ Expand (stack)
 
 /* Callbacks to user interface functions */
 
-static void
-StackCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure;
-    XtPointer	data;
+static CardStackPtr
+WidgetToStack(Widget w, int col)
 {
-    CardsInputPtr    input = (CardsInputPtr) data;
-    CardStackPtr    stack;
-    CardPtr	    card;
-    String	    type;
-    int		    i;
-
-    Message (message, "");
     if (w == stacks)
-	stack = &stackStacks[input->col];
+	return &stackStacks[col];
     else if (w == suits)
-	stack = &suitStacks[input->col];
+	return &suitStacks[col];
     else if (w == piles)
-	stack = &pileStacks[input->col];
-    else
+	return &pileStacks[col];
+    return NULL;
+}
+
+static void
+InputCallback (Widget w, XtPointer closure, XtPointer data)
+{
+    HandInputPtr    input = (HandInputPtr) data;
+    CardStackPtr    stack = NULL;
+    CardStackPtr    startStack = NULL;
+    CardPtr	    card = NULL;
+
+    (void) closure;
+    Message (message, "");
+    stack = WidgetToStack(w, input->current.col);
+    startStack = WidgetToStack(input->start.w, input->start.col);
+
+    if (!startStack || !stack)
 	return;
-    for (card = stack->last; card; card = card->prev)
-	if (card->isUp && card->row == input->row)
-	    break;
-    if (*input->num_params) {
-	type = *input->params;
-	if (!strcmp (type, "source"))
-	{
-	    fromStack = stack;
-	    fromCard = 0;
-	    if (!fromStack->last)
-		Message (message, "Selected stack is empty.");
-	}
-	else if (!strcmp (type, "dest"))
-	{
-	    if (fromStack)
-	    {
-		Play (fromStack, fromCard, stack);
-		fromStack = NULL;
-		CardNextHistory ();
-		DisplayStacks ();
-	    }
-	}
-	else if (!strcmp (type, "expand"))
-	{
-	    Expand (stack);
-	}
+
+    CardSetAnimate(True);
+    switch (input->action) {
+    case HandActionStart:
+        break;
+    case HandActionDrag:
+        if (startStack == stack)
+            break;
+        CardSetAnimate(False);
+        /* fall through */
+    case HandActionClick:
+        card = startStack->last;
+        Play (startStack, card, stack);
+        CardNextHistory ();
+        DisplayStacks ();
+        break;
+    case HandActionExpand:
+        Expand (stack);
+        break;
+    case HandActionUnexpand:
+        break;
     }
 }
 
 static void
-NewGameCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+NewGameCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     NewGame ();
 }
 
 static void
-QuitCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+QuitCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Quit ();
 }
 
 static void
-ScoreCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+ScoreCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Score ();
 }
 
 static void
-UndoCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+UndoCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Undo ();
 }
 
 static void
-FindAMoveCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+FindAMoveCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     FindAMove ();
 }
 
 static void
-RestoreCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+RestoreCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Restore ();
 }
 
 static void
-SaveCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+SaveCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     Save ();
 }
 
 static void
-PileAllCallback (w, closure, data)
-    Widget	w;
-    XtPointer	closure, data;
+PileAllCallback (Widget w, XtPointer closure, XtPointer data)
 {
+    (void) w;
+    (void) closure;
+    (void) data;
     PileAll ();
 }
 
 /* actions to user interface functions */
 
-static void UndoAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void UndoAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Undo ();
 }
-    
-static void NewGameAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void NewGameAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     NewGame ();
 }
-    
-static void ScoreAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void ScoreAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Score ();
 }
-    
-static void QuitAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+
+static void QuitAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Quit ();
 }
 
-static void FindAMoveAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void FindAMoveAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     FindAMove ();
 }
 
-static void RestoreAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void RestoreAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Restore ();
 }
 
-static void SaveAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void SaveAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     Save ();
 }
 
-static void PileAllAction (w, e, p, n)
-    Widget  w;
-    XEvent  *e;
-    String  p;
-    Cardinal*n;
+static void PileAllAction (Widget w, XEvent *e, String *p, Cardinal *n)
 {
+    (void) w;
+    (void) e;
+    (void) p;
+    (void) n;
     PileAll ();
 }
 
 XtActionsRec	actions[] = {
-    "ktowersUndo",	UndoAction,
-    "ktowersNewGame",	NewGameAction,
-    "ktowersScore",	ScoreAction,
-    "ktowersQuit",	QuitAction,
-    "ktowersFindAMove",	FindAMoveAction,
-    "ktowersRestore",	RestoreAction,
-    "ktowersSave",	SaveAction,
-    "ktowersPileAll",	PileAllAction,
+    { "ktowersUndo",	UndoAction, },
+    { "ktowersNewGame",	NewGameAction, },
+    { "ktowersScore",	ScoreAction, },
+    { "ktowersQuit",	QuitAction, },
+    { "ktowersFindAMove",	FindAMoveAction, },
+    { "ktowersRestore",	RestoreAction, },
+    { "ktowersSave",	SaveAction, },
+    { "ktowersPileAll",	PileAllAction, },
 };
 
 struct menuEntry {
     char    *name;
-    void    (*function)();
+    void    (*function)(Widget w, XtPointer closure, XtPointer data);
 };
 
 struct menuEntry fileMenuEntries[] = {
-    "restore", RestoreCallback,
-    "save", SaveCallback,
-    "quit", QuitCallback,
+    { "restore", RestoreCallback, },
+    { "save", SaveCallback, },
+    { "quit", QuitCallback, },
 };
-    
-Widget
-CreateMenu (parent, name, entries, count)
-    Widget  parent;
-    char    *name;
-    struct menuEntry	*entries;
-    int	    count;
+
+static Widget
+CreateMenu (Widget parent, char *name, struct menuEntry *entries, int count)
 {
     Widget  menu;
     Widget  entry;
     int	    i;
 
-    menu = XtCreatePopupShell (name, simpleMenuWidgetClass,
+    menu = XtCreatePopupShell (name, ksimpleMenuWidgetClass,
 			       parent, NULL, ZERO);
     for (i = 0; i < count; i++) {
 	entry = XtCreateManagedWidget (entries[i].name,
-				       smeBSBObjectClass, menu, NULL, ZERO);
+				       ksmeBSBObjectClass, menu, NULL, ZERO);
 	XtAddCallback (entry, XtNcallback, entries[i].function, NULL);
     }
     return menu;
@@ -782,61 +758,59 @@ XtResource resources[] = {
 };
 
 XrmOptionDescRec options[] = {
-    "-smallCards",	"*Cards.smallCards",	XrmoptionNoArg, "True",
-    "-squareCards",	"*Cards.roundCards",	XrmoptionNoArg, "False",
-    "-noanimate",	"*animationSpeed",	XrmoptionNoArg, "0",
-    "-animationSpeed",	"*animationSpeed",	XrmoptionSepArg, NULL
+    { "-smallCards",	"*Cards.smallCards",	XrmoptionNoArg, "True", },
+    { "-squareCards",	"*Cards.roundCards",	XrmoptionNoArg, "False", },
+    { "-noanimate",	"*animationSpeed",	XrmoptionNoArg, "0", },
+    { "-animationSpeed",	"*animationSpeed",	XrmoptionSepArg, NULL },
 };
 
-main (argc, argv)
-    int	    argc;
-    char    **argv;
+int
+main (int argc, char **argv)
 {
-    Arg	args[20];
     Atom wm_delete_window;
 
-    toplevel = XtInitialize (argv[0], "KTowers", options, XtNumber(options),
-			     &argc, argv);
-    
+    toplevel = XkwInitialize ("KTowers", options, XtNumber(options),
+			      &argc, argv, True, defaultResources);
+
     XtGetApplicationResources (toplevel, (XtPointer)&ktowersResources, resources,
 			       XtNumber (resources), NULL, 0);
-    
+
     AnimateSetSpeed (ktowersResources.animationSpeed);
-    
+
     XtAddActions (actions, XtNumber(actions));
 
-    XtOverrideTranslations 
-	(toplevel, 
+    XtOverrideTranslations
+	(toplevel,
 	 XtParseTranslationTable ("<Message>WM_PROTOCOLS: ktowersQuit()"));
     frame = XtCreateManagedWidget ("frame", layoutWidgetClass, toplevel, NULL, 0);
     menuBar = XtCreateManagedWidget ("menuBar", layoutWidgetClass, frame, NULL, 0);
     fileMenuButton = XtCreateManagedWidget ("fileMenuButton",
-					    menuButtonWidgetClass,
+					    kmenuButtonWidgetClass,
 					    menuBar, NULL, ZERO);
-    fileMenu = CreateMenu (fileMenuButton, "fileMenu", 
+    fileMenu = CreateMenu (fileMenuButton, "fileMenu",
 			   fileMenuEntries, XtNumber (fileMenuEntries));
-    newGame = XtCreateManagedWidget ("newGame", commandWidgetClass,
+    newGame = XtCreateManagedWidget ("newGame", kcommandWidgetClass,
 				     menuBar, NULL, ZERO);
     XtAddCallback(newGame, XtNcallback, NewGameCallback, NULL);
-    undo = XtCreateManagedWidget ("undo", commandWidgetClass,
+    undo = XtCreateManagedWidget ("undo", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(undo, XtNcallback, UndoCallback, NULL);
-    hint = XtCreateManagedWidget ("hint", commandWidgetClass,
+    hint = XtCreateManagedWidget ("hint", kcommandWidgetClass,
 				  menuBar, NULL, ZERO);
     XtAddCallback(hint, XtNcallback, FindAMoveCallback, NULL);
-    score = XtCreateManagedWidget ("score", commandWidgetClass,
+    score = XtCreateManagedWidget ("score", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(score, XtNcallback, ScoreCallback, NULL);
-    pileAll = XtCreateManagedWidget ("pileAll", commandWidgetClass,
+    pileAll = XtCreateManagedWidget ("pileAll", kcommandWidgetClass,
 				   menuBar, NULL, ZERO);
     XtAddCallback(pileAll, XtNcallback, PileAllCallback, NULL);
     suits = XtCreateManagedWidget ("suits", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (suits, XtNinputCallback, StackCallback, NULL);
+    XtAddCallback (suits, XtNinputCallback, InputCallback, NULL);
     piles = XtCreateManagedWidget ("piles", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (piles, XtNinputCallback, StackCallback, NULL);
+    XtAddCallback (piles, XtNinputCallback, InputCallback, NULL);
     stacks = XtCreateManagedWidget ("stacks", cardsWidgetClass, frame, NULL, 0);
-    XtAddCallback (stacks, XtNinputCallback, StackCallback, NULL);
-    message = XtCreateManagedWidget ("message", labelWidgetClass, frame, NULL, 0);
+    XtAddCallback (stacks, XtNinputCallback, InputCallback, NULL);
+    message = XtCreateManagedWidget ("message", klabelWidgetClass, frame, NULL, 0);
     srandom (getpid () ^ time ((long *) 0));
     NewGame ();
     XtRealizeWidget (toplevel);
@@ -845,5 +819,7 @@ main (argc, argv)
     (void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
                             &wm_delete_window, 1);
 
+    XkwSetCardIcon(toplevel);
+
     XtMainLoop ();
 }
