@@ -9,12 +9,10 @@
 void
 main(void)
 {
-
-	reg int	i, j, count;
-
 	printf("   %16s -> %5s %5s %4s %s\n", "Card", "cards", "count", "need", "opposite");
-	for (i = 0; i < NUM_CARDS - 1; i++) {
-		for (j = 0, count = 0; j < DECK_SZ; j++)
+	for (int i = 0; i < NUM_CARDS - 1; i++) {
+		int count = 0;
+		for (int j = 0; j < DECK_SZ; j++)
 			if (Deck[j] == i)
 				count++;
 		printf("%2d %16s -> %5d %5d %4d %s\n", i, C_name[i], Numcards[i], count, Numneed[i], C_name[opposite(i)]);
